######################################################################
# Create an application server
# Usage:
#    CreateApplicationServer.py <options>
#
# Options:
#    Required:
#       -serverName        - Specify the name for this application server
#
#    Optional:
#       -startingPort      - Specifies the starting port for this server. Do not use with -portsFile
#       -portsFile         - Specifies a properties file containing the port numbers for this server. Do not use with -startingPort
#       -hostName          - Specifies the host name for this server


import sys
import traceback
from java.io import FileInputStream
from java.lang import System
from java.util import Properties



try:
	NL = System.getProperty('line.separator')
	PORTS                  = ['WC_defaulthost','WC_adminhost','WC_defaulthost_secure','WC_adminhost_secure','BOOTSTRAP_ADDRESS','SOAP_CONNECTOR_ADDRESS','SAS_SSL_SERVERAUTH_LISTENER_ADDRESS','CSIV2_SSL_SERVERAUTH_LISTENER_ADDRESS','CSIV2_SSL_MUTUALAUTH_LISTENER_ADDRESS','ORB_LISTENER_ADDRESS','DCS_UNICAST_ADDRESS','SIB_ENDPOINT_ADDRESS','SIB_ENDPOINT_SECURE_ADDRESS','SIB_MQ_ENDPOINT_ADDRESS','SIB_MQ_ENDPOINT_SECURE_ADDRESS','SIP_DEFAULTHOST','SIP_DEFAULTHOST_SECURE']
	DEFAULT_HOST_ENDPOINTS = ['WC_defaulthost','WC_defaulthost_secure','SIP_DEFAULTHOST','SIP_DEFAULTHOST_SECURE']
	ADMIN_HOST_ENDPOINTS   = ['WC_adminhost'  ,'WC_adminhost_secure']
	
	nodes = AdminConfig.list("Node").split(NL)
	node = nodes[0]
	nodeName = AdminConfig.showAttribute(node, "name")
	
	# Get server name
	serverName = None
	try:
		serverName = sys.argv[sys.argv.index('-serverName') + 1]
	except:
		print '-serverName is required'
		sys.exit(1)
	#endtry
	
	# Get ports file, or the starting port
	ports = None
	try:
		ports = Properties()
		ports.load(FileInputStream(sys.argv[sys.argv.index('-portsFile') + 1]))
	except:
		try:
			ports = Properties()
			startingPort = int(sys.argv[sys.argv.index('-startingPort') + 1])
			for portName in PORTS:
				ports.setProperty(portName, str(startingPort))
				startingPort = startingPort + 1
			#endfor
		except:
			ports = None
		#endtry
	#endtry
	
	# Get the host name
	hostName = None
	try:
		hostName = sys.argv[sys.argv.index('-hostName') + 1]
	except:
		hostName = None
	#endtry
	
	
	# Create the server
	if ports is None:
		server = AdminTask.createApplicationServer(nodeName, ["-name", serverName, "-genUniquePorts", "true"])
	else:
		server = AdminTask.createApplicationServer(nodeName, ["-name", serverName])
	#endif
	
	# Get the resulting server entry
	serverEntry = AdminConfig.getid("/ServerEntry:" + serverName)
	
	# Get the virtual host default_host
	defaultHost = AdminConfig.getid("/VirtualHost:default_host")
	
	# Get the virtual host admin_host
	adminHost = AdminConfig.getid("/VirtualHost:admin_host")
	
	# Iterate over all the named end points (ports) for this server
	for namedEndPoint in AdminConfig.list("NamedEndPoint", serverEntry).split(NL):
		# Get the name and endPoint object
		name     = AdminConfig.showAttribute(namedEndPoint, "endPointName")
		endPoint = AdminConfig.showAttribute(namedEndPoint, "endPoint")
		host     = AdminConfig.showAttribute(endPoint,      "host")
		port     = AdminConfig.showAttribute(endPoint,      "port")
		
		if ports is not None and ports.getProperty(name) is not None:
			# Store the port for this end point
			port = ports.getProperty(name)
			
			# Change the end point's port
			AdminConfig.modify(endPoint, [["port", port]])
		#endif
		
		if hostName is not None and host != '*':
			# Get the new host
			host = hostName

			# Change the end point's host
			AdminConfig.modify(endPoint, [["host", host]])
		#endif
		
		# Add a virtual host alias if applicable
		if name in DEFAULT_HOST_ENDPOINTS:
			AdminConfig.create("HostAlias", defaultHost, [["hostname",host], ["port",port]])
		elif name in ADMIN_HOST_ENDPOINTS:
			AdminConfig.create("HostAlias", adminHost,   [["hostname",host], ["port",port]])
		#endif
	#endfor
	
	AdminConfig.save()
except:
	traceback.print_exc();
	sys.exit(1)
#endtry



sys.exit(0)
