######################################################################
# Define imports
import java
import jarray
import re
import os
import sys
import traceback
import types
import pprint
import getopt
import shutil
from java.io import BufferedReader
from java.io import File
from java.io import InputStreamReader
from java.io import FileInputStream
from java.net import URL
from java.net import URLClassLoader
from java.lang import System
from java.lang import Runtime
from java.lang import Thread
from java.text import MessageFormat
from java.util import Locale
from java.util import Properties
from java.util import ResourceBundle





######################################################################
# Get the system separators
global NL
NL = System.getProperty('line.separator')

global PS
PS = System.getProperty('path.separator')

global FS
FS = System.getProperty('file.separator')





######################################################################
# Define a pretty printer for nice object printing
global PPRINTER
PPRINTER = pprint.PrettyPrinter()
def pprint(value):
	"""Print the given value using PrettyPrinter."""
	
	passwordKeys = ['defaultPassword','password','mailTransportPassword','mailStorePassword']
	passwordStore = {}
	if isinstance(value, types.DictType):
		for passwordKey in passwordKeys:
			if value.has_key(passwordKey):
				passwordStore[passwordKey] = value[passwordKey]
				value[passwordKey] = '********'
			#endif
		#endfor
	#endif
	
	PPRINTER.pprint(value)
	print ''
	
	for passwordKey in passwordKeys:
		if passwordStore.has_key(passwordKey):
			value[passwordKey] = passwordStore[passwordKey]
		#endif
	#endfor
#enddef





def printBanner(str):
	"""Print the given string as a banner."""
	print ''
	print '######################################################################'
	print str
	print '######################################################################'
	print ''
#enddef





######################################################################
# Load the options provided to this profile
global OPTIONS
OPTIONS = {}
try:
	OPTIONS, REMAINING = getopt.getopt(sys.argv, 'o:', ['options='])
	sys.argv = REMAINING
except:
	print 'Invalid invocation:'
	pprint(sys.argv)
	sys.exit(2)
#endtry





######################################################################
# Load the properties file specified by the argument --options or -o
global PROPERTIES
PROPERTIES = Properties()
for option, value in OPTIONS:
	if option in ('--options', '-o'):
		try:
			PROPERTIES.load(FileInputStream(File(value)))
		except:
			print >>sys.stderr, 'Could not load properties from the properties file ' + value + '.'; traceback.print_exc();
			sys.exit(2)
		#endtry
		break
	#endif
#endfor





def PROP(key, default = None):
	"""Return the property stored under the given key."""
	value = PROPERTIES.getProperty(key, default)
	if value is None:
		print MESSAGE('property_error', key)
		value = ''
	#endif
	return value
#enddef





######################################################################
# Get the root directories
global PROFILE_ROOT
PROFILE_ROOT = System.getProperty('user.install.root') + FS

global ROOT
ROOT = PROP('launcher.root', '.') + FS

global CLASSPATH
CLASSPATH = [ROOT]
for path in PROP('launcher.classpath', '').split(PS):
	if(len(path) > 0):
		CLASSPATH.append(path)
	#endif
#endfor





######################################################################
# Load the translated messages resource bundle
global MESSAGES
MESSAGES = None
try:
	urls = []
	for path in CLASSPATH:
		urls.append(File(path).toURL())
	#endfor
	classloader = URLClassLoader(jarray.array(urls, URL))
	MESSAGES = ResourceBundle.getBundle('com.ibm.jsdt.support.webapp.Messages', Locale.getDefault(), classloader)
except:
	print >>sys.stderr, 'Could not load the message properties file.'; traceback.print_exc();
#endtry





def MESSAGE(key, *tokens):
	"""Return the translated string for the given message key, with the given tokens inserted."""
	value = key;
	if MESSAGES is not None:
		value = MESSAGES.getString(key)
		if value is None:
			value = key
		#endif
	#endif
	return MessageFormat.format(value, tokens)
#enddef





######################################################################
# Get OS-specific settings
global OS_WINDOWS
global OS_LINUX
global OS_OS400
global OS_AIX
global OS_HPUX
global OS_SUN
OS_WINDOWS = 'win32'
OS_LINUX   = 'linux'
OS_OS400   = 'os400'
OS_AIX     = 'aix'
OS_HPUX    = 'hpux'
OS_SUN     = 'solaris'

global ARCH_X86
global ARCH_PPC
global ARCH_PA_RISC
ARCH_X86 = 'x86'
ARCH_PPC = 'ppc'
ARCH_PA_RISC = 'PA_RISC'
ARCH_SPARC = 'sparc'

global OS
global ARCH

if System.getProperty('os.name').find('Windows') >= 0:
    OS               = OS_WINDOWS
    ARCH             = ARCH_X86
elif System.getProperty('os.name').find('OS/400') >= 0:
    OS               = OS_OS400
    ARCH             = ARCH_PPC
elif System.getProperty('os.name').find('Linux') >= 0:
    OS               = OS_LINUX
    if System.getProperty('os.arch').find('ppc') >= 0:
        ARCH = ARCH_PPC
    else:
        ARCH = ARCH_X86
    #endif
elif System.getProperty('os.name').find('AIX') >= 0:
    OS               = OS_AIX
    ARCH             = ARCH_PPC
elif System.getProperty('os.name').find('HP-UX') >= 0:
    OS               = OS_HPUX
    ARCH             = ARCH_PA_RISC
elif System.getProperty('os.name').find('SunOS') >= 0:
    OS               = OS_SUN
    if System.getProperty('os.arch').find('sparc') >= 0:
        ARCH = ARCH_SPARC
    else:
        ARCH = ARCH_X86
    #endif     
else:
	print MESSAGE('os_error', System.getProperty('os.name'))
	sys.exit(1)
#endif





######################################################################
# Make sure we have a handle to AdminTask, AdminConfig, AdminApp and AdminControl
try:
	AdminTask
	AdminConfig
	AdminApp
	AdminControl
except:
	print MESSAGE('ensure_server_started_and_correct_authentication')
	sys.exit(1)
#endtry





######################################################################
# Define True and False constants for older versions of Python
try:
	True, False
except NameError:
	(True, False) = (1, 0)
#endtry





def parseList(list):
	"""Parse and return a list of strings from the output of AdminConfig.list().
	
	list -- A string containing the output of a call to AdminConfig.list()
	"""
	if not list:
		return []
	else:
		return map(trimQuotes, list.strip().split(NL))
	#endif
#enddef





def parseShow(show):
	"""Parse and return a dictionary of attribute=>value mappings from the output of AdminConfig.show(...).
	
	show -- A string containing the output of a call to AdminConfig.show(...)
	"""
	showMap = {}
	if not show:
		return showMap
	else:
		lines = show.strip().split(NL)
		for line in lines:
			match = re.match(r'\[([^ ]+) (.*)\]', line.strip())
			if match:
				key = match.group(1)
				value = trimQuotes(match.group(2))
				showMap[key] = value
			#endif
		#endfor
		return showMap
	#endif
#enddef





def trimQuotes(val):
	"""Trim single or double quotes off the beginning and end of the given string."""
	match = re.match('[\'\"](.*)[\'\"]', val)
	if match:
		return match.group(1)
	else:
		return val
	#endif
#enddef





def dictToArgs(dict):
	"""Convert the given dictionary to an arguments array in the form [[key, value], [key, value], ...]."""
	args = []
	if dict is not None:
		for key in dict.keys():
			if key != 'scope' and dict[key] is not None:
				args.append([key, dict[key]])
			#endif
		#endfor
	#endif
	return args
#enddef





def dictToSwitches(dict):
	"""Convert the given dictionary to an arguments string in the form "-key 'value' -key 'value' ..."."""
	args = ''
	if dict is not None:
		for key in dict.keys():
			if key != 'scope' and dict[key] is not None:
				args += "-%s '%s' " % (key, dict[key])
			#endif
		#endfor
	#endif
	return args
#enddef





def getKeyOrSelf(value, key):
	"""If the given value is a dictionary, return the specified keyed value, otherwise return the value itself.
	
	value -- dictionary or string
	key -- The key to use to look up the return value if the given value is a dictionary
	"""
	if isinstance(value, types.DictType):
		if value.has_key(key):
			value = value[key]
		else:
			value = None
		#endif
	#endif
	return value
#enddef





def getScope(scope = None):
	"""Return the scope of the given object, suitable for passing to AdminConfig.show()."""
	return getKeyOrSelf(scope, 'scope')
#enddef





def getName(name = None):
	"""Return the name of the given object if it has one."""
	return getKeyOrSelf(name, 'name')
#enddef





def getID(scope = None):
	"""Return the XMI ID of the given object."""
	match = re.match(r'^.*#([^#]*)\)$', getScope(scope))
	if match:
		return match.group(1)
	else:
		return None
	#endif
#enddef





def getCell(scope = None):
	"""Return the parent cell of the given object."""
	match = re.match(r'^.*cells/([^/]*)(/|\|).*$', getScope(scope))
	if match:
		return match.group(1)
	else:
		return None
	#endif
#enddef





def getNode(scope = None):
	"""Return the parent node of the given object."""
	match = re.match(r'^.*/nodes/([^/]*)(/|\|).*$', getScope(scope))
	if match:
		return match.group(1)
	else:
		return None
	#endif
#enddef





def getServer(scope = None):
	"""Return the parent server of the given object."""
	match = re.match(r'^.*/servers/([^/]*)(/|\|).*$', getScope(scope))
	if match:
		return match.group(1)
	else:
		return None
	#endif
#enddef





def getResource(type, conditions = None, scope = None):
	"""Return the first resource of the given type that matches the given conditions in the given scope.
	
	type -- WebSphere resource type, as accepted by AdminConfig.list()
	conditions -- A dictionary of attribute:value pairs, which must match what is returned by AdminConfig.show() for a given object
	scope -- The parent scope to limit the search to.
	"""
	if conditions is None:
		conditions = {}
	#endif
	
	if not isinstance(conditions, types.DictType):
		print MESSAGE('conditions_error')
		return None
	#endif
	
	retval = None
	for resource in parseList(AdminConfig.list(type, getScope(scope))):
		potential = parseShow(AdminConfig.show(resource))
		potential['scope'] = resource
		potential['xmi:id'] = getID(resource)
		
		satisfied = True
		for key in conditions.keys():
			if (not potential.has_key(key)) or potential[key] != conditions[key]:
				satisfied = False
				break
			#endif
		#endfor
		
		if satisfied and (retval is None or len(retval['scope']) > len(potential['scope'])):
			retval = potential
		#endif
	#endfor
	return retval
#enddef





def createResource(type, parent, args = None, key = None):
	"""
	Create or modify a configuration resource.
	
	type -- The type of resource to create, as accepted by AdminConfig
	parent -- The parent scope to create this resource under
	args -- A dictionary of attributes
	key -- The key for the value in the args dictionary that uniquely identifies this resource
	"""
	try:
		# Initialize the condition to match an existing resource
		condition = None
		
		if key is None or args is None:
			printBanner(MESSAGE('configuring_unnamed_resource', type));
		else:
			printBanner(MESSAGE('configuring_named_resource', type, args[key]));
			condition = {key:args[key]}
		
		print MESSAGE('parent')
		pprint(getScope(parent))
		
		print MESSAGE('arguments')
		pprint(args)
		
		# Get the resource if it already exists
		existing = getResource(type, condition, parent)
		
		if existing is None:
			print MESSAGE('creating_resource', type)
			AdminConfig.create(type, getScope(parent), dictToArgs(args))
		else:
			print MESSAGE('modifying_resource', type, getScope(existing))
			AdminConfig.modify(getScope(existing), dictToArgs(args))
		#endif
		
		# Get the resulting resource
		existing = getResource(type, condition, parent)
		print ''
		print MESSAGE('result')
		pprint(existing)
		return existing
	except:
		print >>sys.stderr, MESSAGE('configuration_error', type)
		traceback.print_exc()
		sys.exit(1)
	#endtry
	return None
#enddef





def createLibraryResource(type, parent, args, key, libraryPath, classpathName, nativepathName):
	"""Create or modify a library resource.
	
	The classpath and nativepath attributes are dynamically determined by looking in the given libraryPath.
	If a subfolder with the same name as the resource is found, any JAR files within that folder are included 
	in the classpath.
	
	If a lib/OS/ARCH subfolder exists within the folder, the nativepath entry is set to point to that folder.
	
	type -- The type of resource to create, as accepted by AdminConfig.
	parent -- The parent scope to create the resource under
	args -- A dictionary of attributes
	key -- The key for the value in the args dictionary that uniquely identifies this resource
	libraryPath -- The folder containing classpath and nativepath files for this resource
	"""
	try:
		args[classpathName] = ''
		args[nativepathName] = ''
		libraryDir = File(libraryPath)
		if libraryDir.isDirectory():
			args[classpathName] = buildClasspathString(libraryPath)
			
			nativeDir = File(libraryDir, 'lib' + FS + OS + FS + ARCH)
			if nativeDir.isDirectory() and nativeDir.listFiles():
				args[nativepathName] = nativeDir.getAbsolutePath().replace(FS, '/')
			#endif
		#endif
	except:
		print >>sys.stderr, MESSAGE('library_setup_error', type, args[key])
		traceback.print_exc()
		sys.exit(1)
	#endtry
	return createResource(type, parent, args, key)

#enddef





def buildClasspathString(path = None):
	"""Build a ;-separated classpath string from the jar files in the given directory.
	
	path -- The directory to scan for jar files
	"""
	children = (path and File(path).listFiles()) or []
	jars = filter(lambda x: x.isDirectory or (x.isFile() and x.getName().lower().endswith('.jar')), children)
	paths = map(lambda x: x.getAbsolutePath().replace(FS, '/'), jars)
	return ";".join(paths) + ';'
#enddef





def createJDBCProvider(type, parent, args = None, key = None):
	"""Create or modify a JDBC provider resource.
	
	type -- The type of resource to create, as accepted by AdminConfig. Should be "JDBCProvider"
	parent -- The parent scope to create the JDBC provider under
	args -- A dictionary of attributes
	key -- The key for the value in the args dictionary that uniquely identifies this JDBC provider
	"""
	
	return createLibraryResource(type, parent, args, key, PROFILE_ROOT + 'optionalLibraries' + FS + args['name'], 'classpath', 'nativepath')
#enddef





def createSharedLibrary(type, parent, args = None, key = None):
	"""Create or modify a Library resource.
	
	type -- The type of resource to create, as accepted by AdminConfig. Should be "Library"
	parent -- The parent scope to create the shared library under
	args -- A dictionary of attributes
	key -- The key for the value in the args dictionary that uniquely identifies this library
	"""
	
	return createLibraryResource(type, parent, args, key, PROFILE_ROOT + 'sharedLibraries' + FS + args['name'], 'classPath', 'nativePath')
#enddef





def adminTaskCall(type, name, parent, condition, args, createMethod, modifyMethod):
	"""Create or modify an object using a call to an AdminTask method.
	
	type -- The type of resource to create, as accepted by AdminConfig.
	parent -- The parent scope to create the resource under
	args -- A dictionary of attributes
	key -- The key for the value in the args dictionary that uniquely identifies this resource
	createMethod -- A function pointer to call to create a new instance of this resource. Must match the signature createMethod(parentObject, existingObject, args)
	modifyMethod -- A function pointer to call to modify an existing instance of this resource. Must match the signature modifyMethod(parentObject, existingObject, args)
	"""
	try:
		printBanner(MESSAGE('configuring_named_resource', type, name));
		
		print MESSAGE('parent')
		pprint(getScope(parent))
		
		print MESSAGE('arguments')
		pprint(args)
		
		# Get the resource if it already exists
		existing = getResource(type, condition, parent)
		
		if existing is None:
			if createMethod is None:
				print MESSAGE('resource_does_not_exist', name)
			else:
				print MESSAGE('creating_resource', type)
				createMethod(parent, existing, args)
			#endif
		else:
			if modifyMethod is None:
				print MESSAGE('resource_exists', name)
			else:
				print MESSAGE('modifying_resource', type, getScope(existing))
				modifyMethod(parent, existing, args)
			#endif
		#endif
		
		# Get the resulting resource
		existing = getResource(type, condition, parent)
		print ''
		print MESSAGE('result')
		pprint(existing)
		return existing
	except:
		print >>sys.stderr, MESSAGE('configuration_error', type); traceback.print_exc()
		sys.exit(1)
	#endtry
	return None
#enddef





def createSIBJMSActivationSpec(type, parent, args = {}, key = None):
	"""Create or update a resource of type SIBJMSActivationSpec."""
	condition = {key : args[key]}
	def create(parent, existing, args):
		AdminTask.createSIBJMSActivationSpec(getScope(parent), dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		AdminTask.modifySIBJMSActivationSpec(getScope(existing), dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args[key], parent, condition, args, create, modify)
#enddef





def createSIBDestination(type, parent, args = {}, key = None):
	"""Create or update a resource of type SIBDestination."""
	condition = {key : args['name']}
	def create(parent, existing, args):
		AdminTask.createSIBDestination(dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		if args is not None:
			args['type'] = None
			args['useRFH2'] = None
			args['aliasBus'] = None
			args['foreignBus'] = None
			args['targetBus'] = None
			args['targetName'] = None
			args['cluster'] = None
			args['node'] = None
			args['server'] = None
			args['wmqServer'] = None
			args['wmqQueueName'] = None
			args['nonPersistentReliability'] = None
			args['persistentReliability'] = None
		#endif
		AdminTask.modifySIBDestination(dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args['name'], parent, condition, args, create, modify)
#enddef





def createSIBJMSConnectionFactory(type, parent, args = {}, key = None):
	"""Create or update a resource of type SIBJMSConnectionFactory."""
	condition = {key : args[key]}
	def create(parent, existing, args):
		AdminTask.createSIBJMSConnectionFactory(getScope(parent), dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		if args is not None:
			args['targetSignificance'] = None
			args['type'] = None
		#endif
		AdminTask.modifySIBJMSConnectionFactory(getScope(existing), dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args[key], parent, condition, args, create, modify)
#enddef





def createSIBJMSQueue(type, parent, args = {}, key = None):
	"""Create or update a resource of type SIBJMSQueue."""
	condition = {key : args[key]}
	def create(parent, existing, args):
		AdminTask.createSIBJMSQueue(getScope(parent), dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		AdminTask.modifySIBJMSQueue(getScope(existing), dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args[key], parent, condition, args, create, modify)
#enddef





def createSIBJMSTopic(type, parent, args = {}, key = None):
	"""Create or update a resource of type SIBJMSTopic."""
	condition = {key : args[key]}
	def create(parent, existing, args):
		AdminTask.createSIBJMSTopic(getScope(parent), dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		AdminTask.modifySIBJMSTopic(getScope(existing), dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args[key], parent, condition, args, create, modify)
#enddef





def createSIBus(type, parent, args = {}, key = None):
	"""Create or update a service integration bus."""
	condition = {key : args['bus']}
	def create(parent, existing, args):
		AdminTask.createSIBus(dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		AdminTask.modifySIBus(dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args['bus'], parent, condition, args, create, modify)
#enddef





def createSIBMQServer(type, parent, args = {}, key = None):
	"""Create or update an MQ server."""
	condition = {key : args[key]}
	def create(parent, existing, args):
		AdminTask.createSIBWMQServer(dictToSwitches(args))
	#enddef
	def modify(parent, existing, args):
		if args is not None:
			args['type'] = None
		#endif
		AdminTask.modifySIBWMQServer(dictToSwitches(args))
	#enddef
	return adminTaskCall(type, args[key], parent, condition, args, create, modify)
#enddef





def createSIBusMember(type, parent, args = {}, key = None):
	"""Create an SIBusMember."""
	condition = {}
	name = ''
	if args.has_key('node'):
		name += args['node'] + '/'
		condition['node'] = args['node']
	#endif
	if args.has_key('server'):
		name += args['server']
		condition['server'] = args['server']
	#endif
	if args.has_key('wmqServer'):
		name = args['wmqServer']
		condition['mqServer'] = args['wmqServer']
	#endif
	
	def create(parent, existing, args):
		AdminTask.addSIBusMember(dictToSwitches(args))
	#enddef
	
	return adminTaskCall(type, name, parent, condition, args, create, None)
#enddef





def applicationExists(name, cell, node, server):
	"""Return true if an application with the given name exists in the given scope.
	
	name -- The application name to look for.
	cell -- The cell name to look for the application in.
	node -- The node name to look for the application in.
	server -- The server name to look for the application in.
	"""
	return name in parseList(AdminApp.list('WebSphere:cell=%s,node=%s,server=%s' % (getName(cell), getName(node), getName(server))))
#enddef





def installApplication(name, file, args, cell, node, server, classloaderMode = None, warClassLoaderPolicy = None):
	"""Install or update the application with the given name in the given scope.
	
	name -- The application name.
	file -- The full path to the application EAR file.
	args -- An array of arguments to pass to AdminApp.install or AdminApp.update
	cell -- The cell scope to install/update the application in.
	node -- The node scope to install/update the application in.
	server -- The application server to install/update the application in.
	classloaderMode -- PARENT_LAST or PARENT_FIRST
	warClassLoaderPolicy -- SINGLE or MULTIPLE
	"""
	try:
		cell = getName(cell)
		node = getName(node)
		server = getName(server)
		args.extend(['-cell', cell])
		args.extend(['-node', node])
		args.extend(['-server', server])

		if applicationExists(name, cell, node, server):
			printBanner(MESSAGE('updating_application', name))
			args.extend(['-operation', 'update'])
			args.extend(['-contents', file])
			print MESSAGE('arguments')
			pprint(args)
			AdminApp.update(name, 'app', args)
		else:
			printBanner(MESSAGE('installing_application', name))
			print MESSAGE('arguments')
			pprint(args)
			AdminApp.install(file, args)
		#endif
		
		deployedObject = AdminConfig.showAttribute(AdminConfig.getid('/Deployment:' + name), 'deployedObject')
		if classloaderMode is not None:
			print MESSAGE('updating_classloader', classloaderMode)
			classloader = AdminConfig.showAttribute(deployedObject, 'classloader')
			AdminConfig.modify(classloader, [['mode', classloaderMode]])
		#endif
		if warClassLoaderPolicy is not None:
			print MESSAGE('updating_war_policy', warClassLoaderPolicy)
			AdminConfig.modify(deployedObject, [['warClassLoaderPolicy', warClassLoaderPolicy]])
		#endif
	except:
		print >>sys.stderr, MESSAGE('application_installation_error', name)
		traceback.print_exc()
		sys.exit(1)
	#endtry
#enddef





def startApplication(name, cell, node, server):
	"""Start the given application.
	
	name -- The application name.
	cell -- The cell name to look for the application in.
	node -- The node name to look for the application in.
	server -- The server name to look for the application in.
	"""
	cell = getName(cell)
	node = getName(node)
	server = getName(server)

	printBanner(MESSAGE('starting_application', name))
	try:
		appMBean = AdminControl.completeObjectName('type=Application,name=%s,cell=%s,node=%s,process=%s,*' % (name, cell, node, server))
		if appMBean == '':
			appManager = AdminControl.queryNames('type=ApplicationManager,cell=%s,node=%s,process=%s,*' % (cell, node, server))
			AdminControl.invoke(appManager, 'startApplication', name)
		else:
			print MESSAGE('application_already_started')
		#endif
	except:
		print >>sys.stderr, MESSAGE('application_start_error', name)
		traceback.print_exc()
		sys.exit(1)
	#endtry
#enddef





def save():
	"""Save the configuration."""
	try:
		print MESSAGE('saving'),
		AdminConfig.save()
		print MESSAGE('done')
		return 0
	except:
		print >>sys.stderr, MESSAGE('save_error')
		traceback.print_exc()
		sys.exit(1)
	#endtry
#enddef
